"use strict";

/* eslint-disable flowtype/require-valid-file-annotation */
var buildDevExpression = require('./buildDevExpression');
/**
 * import { invariant } from "@kiwicom/js";
 * invariant(condition, argument, argument);
 *
 *      ↓ ↓ ↓ ↓ ↓ ↓
 *
 * import { invariant } from "@kiwicom/js";
 * if (!condition) {
 *   if (process.env.NODE_ENV !== "production") {
 *     invariant(false, argument, argument);
 *   } else {
 *     invariant(false);
 *   }
 * }
 *
 * This should then be minified by dead code elimination.
 */


module.exports = function (babel) {
  var SEEN_SYMBOL = Symbol('SEEN_SYMBOL');
  return {
    pre: function pre() {
      this.canPerformChanges = false;
    },
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@kiwicom/js') {
          this.canPerformChanges = true;
        }
      },
      CallExpression: {
        enter: function enter(path) {
          if (path.get('callee').isIdentifier({
            name: 'require'
          })) {
            if (path.node.arguments[0].value === '@kiwicom/js') {
              this.canPerformChanges = true;
            }
          }
        },
        exit: function exit(path) {
          var node = path.node; // do nothing when testing

          if (process.env.NODE_ENV === 'test') {
            return;
          } // not our function


          if (this.canPerformChanges !== true) {
            return;
          } // ignore if it's already been processed


          if (node[SEEN_SYMBOL]) {
            return;
          }

          if (path.get('callee').isIdentifier({
            name: 'invariant'
          })) {
            var prodInvariant = babel.template.expression.ast('invariant(false)');
            var devInvariant = babel.template.expression('invariant(false, %%arguments%%)')({
              arguments: node.arguments.slice(1)
            });
            prodInvariant[SEEN_SYMBOL] = true;
            devInvariant[SEEN_SYMBOL] = true;
            path.replaceWith(babel.template("\n                if (!%%condition%%) {\n                  if (%%innnerCondition%%) {\n                    %%devInvariant%%\n                  } else {\n                    %%prodInvariant%%\n                  }\n                }\n              ")({
              condition: node.arguments[0],
              innnerCondition: buildDevExpression(babel),
              prodInvariant: prodInvariant,
              devInvariant: devInvariant
            }));
          }
        }
      }
    }
  };
};